# -*- coding: utf-8 *-*

import requests


class ApiError(Exception):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ipx800:
    def __init__(self, url, api_key="apikey"):
        self.url = url
        self._api_url = f"{url}/api/xdevices.json"
        self.api_key = api_key

    def _send_command(self, params):
        # (bug) IPX4, key must be the first parameter otherwise some
        # calls don't return.
        # params.update({"key": self.api_key})
        params_fix = {"key": self.api_key}
        params_fix.update(params)
        r = requests.get(self._api_url, params=params_fix, timeout=2)
        r.raise_for_status()
        result = r.json()
        if result["status"] == "Success":
            return result
        else:
            raise ApiError()


class relay(ipx800):
    def __init__(self, ipx, relay_id):
        super().__init__(ipx.url, ipx.api_key)
        self.id = relay_id

    @property
    def status(self):
        params = {"Get": "R"}
        response = self._send_command(params)
        return response[f"R{self.id}"] == 1

    def on(self):
        params = {"SetR": self.id}
        result = self._send_command(params)
        return result["status"] == "Success"

    def off(self):
        params = {"ClearR": self.id}
        result = self._send_command(params)
        return result["status"] == "Success"

    def toggle(self):
        params = {"ToggleR": self.id}
        response = self._send_command(params)
        return response[f"R{self.id}"] == 1

    def __repr__(self):
        return f"<ipx800.relay id={self.id}>"

    def __str__(self):
        return f"[IPX800-relay: id={self.id}, status={self.status}"



# coding: utf-8

import json
from unittest import TestCase
from unittest.mock import Mock, call, patch

from ipx800 import ApiError, ipx800, relay


class IPX800Test(TestCase):
    def setUp(self):
        with open("tests/clearr2.json", "r", encoding="utf-8") as f:
            self.sample_clear_r2 = json.loads(f.read())
        with open("tests/error.json", "r", encoding="utf-8") as f:
            self.sample_error = json.loads(f.read())
        with open("tests/getall.json", "r", encoding="utf-8") as f:
            self.sample_get_all = json.loads(f.read())
        with open("tests/getr.json", "r", encoding="utf-8") as f:
            self.sample_get_r = json.loads(f.read())
        with open("tests/toggler.json", "r", encoding="utf-8") as f:
            self.sample_toggle_r = json.loads(f.read())

    def _mock_response(
        self,
        status=200,
        content="CONTENT",
        json_data=None,
        raise_for_status=None,
    ):
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        if raise_for_status:
            mock_response.raise_for_status.side_effect = raise_for_status
        mock_response.status_code = status
        mock_response.content = content
        if json_data:
            mock_response.json = Mock(return_value=json_data)
        return mock_response

    @patch("requests.get")
    def test_invalid_relay(self, mock_request):
        mock_request.return_value = self._mock_response(
            json_data=self.sample_error
        )

        ipx = ipx800("http://192.0.2.4")
        r999 = relay(ipx, 999)
        with self.assertRaises(ApiError):
            r999.status
        self.assertIn(
            call(
                "http://192.0.2.4/api/xdevices.json",
                params={"key": "apikey", "Get": "R"},
                timeout=2,
            ),
            mock_request.call_args_list,
        )
        self.assertEqual(len(mock_request.call_args_list), 1)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['ipx800']

package_data = \
{'': ['*']}

install_requires = \
['requests']

setup(name='gce-ipx800',
      version='0.2',
      description='Module to control the IPX800 V4 device from GCE Electronics.',
      author='Marc-Aurèle Brothier',
      author_email='m@brothier.org',
      url='https://github.com/marcaurele/py-ipx800',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.6',
     )
